#!/bin/sh
#
# This script pre-processes daily or monthly FINN emissions for use in GEOS-Chem.
# 1. Run exteneral fortran program fire_emis to get CO2 emissions
# 2. Remove date record to allow longitude shifting
# 3. Change time units to TAU0 and shift longitude to -180:180
# 4. Rename category
# 5. Change units
# 6. Merge different vegetation types into 1 file
# 7. Compress output

# REQUIRES:
# perl script changedate.pl
# fortran fire_emis code
# FINN data files
# NCO & CDO utilities
# input file template: fire_emis.inp-template
#   use this one to specify data file location & name

# VARIABLES:
# Set TIMERES (daily or monthly), year

# For questions or more information:
#   Jenny Fisher, jennyf@uow.edu.au

# Load NCO, CDO utilities
module load cdo
module load nco

# User defined variables
TIMERES="daily"
YEAR="2007"

# Start & enddate (these can be changed as needed)
sd=${YEAR}"-01-01"
ed=${YEAR}"-01-31"

# Echo info to screen
echo "Processing ${TIMERES} FINN emissions for:" ${sd} "-" ${ed}

# Vegetation types
VEGTYPES=(1 2 3 4 5 9)

# Filename info
PREFIX1="emissions_CO2_vegtype"
PREFIX2=$TIMERES"_CO2_vegtype"
SUFFIX1="_0.5x0.5.nc"
SUFFIX2=$YEAR"_05x05.nc"
OUTFILENAME="FINN_"$TIMERES"_"$YEAR"_05x05.nc"

# Loop over vegetation types for individual files
for i in "${VEGTYPES[@]}"; do

    echo "  Processing vegtype " $i
    THISFILE_IN=$PREFIX1$i$SUFFIX1
    THISFILE_OUT=$PREFIX2$i"_"$SUFFIX2
    THISVAR="fire_vegtype"$i

    # Set up input file for fire_emis code
    perl set_inputfile.pl fire_emis.inp-template ${sd} ${ed} ${TIMERES} $i 

    # Run fire_emis code
    ./fire_emis/src/fire_emis < fire_emis.inp

    # Remove date record since it messes up longitude shifting
    ncks -x -O -v date ${THISFILE_IN} tmp1.nc

    # Add gamap category & change units (actual unit conversion below)
    ncatted -a gamap_category,fire,c,c,"BIOBSRCE" -a units,fire,o,c,kg/m2/s tmp1.nc

    # Rename variable 'fire' to 'fire_typeX'
    ncrename -h -O -v fire,${THISVAR} tmp1.nc tmp2.nc

    # Shift time axis to hours since 1985-01-01 (TAU0), shift longitude axis, convert units (molec CO2/cm2/s to kg CO2/m2/s)
    cdo mulc,7.30654e-22 -sellonlatbox,-179.75,179.75,-89.75,89.75 -setreftime,1985-01-01,00:00:00,hours tmp2.nc ${THISFILE_OUT}

done

# Merge vegtype files (and compress...?)
#  Note: on some systems you can compress using CDO.
#  To do so use this line instead of the next:
#  cdo -f nc4c -z zip merge ${PREFIX2}*.nc ${OUTFILENAME}
cdo -f nc merge ${PREFIX2}*.nc ${OUTFILENAME}

# Remove intermediate files
rm -f tmp*nc
rm -f ${PREFIX1}*.nc
rm -f ${PREFIX2}*.nc

exit
